@@ -676,6 +676,9 @@ pub mod credentials {
         make_test!(ecs_assume_role);
         make_test!(ecs_credentials);
 
+        make_test!(sso_assume_role);
+        make_test!(sso_no_token_file);
+
         #[tokio::test]
         async fn profile_name_override() {
             let (_, conf) =
